import { Synchronizer } from "../synchronizer.interface";
/**
 * A mutex to manage "concurrency in Javascript" (as mentioned in the mozilla documentation).
 *
 * Implementation's note: the mutex simply use an internal semaphore.
 *
 * @see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise#promise_concurrency
 */
export declare class Mutex implements Synchronizer {
    /**
     * The internal semaphore for mutex implementation
     */
    private readonly semaphore;
    /**
     * @returns if the current mutex is currently locked
     */
    get isLocked(): boolean;
    /**
     * @inheritDoc
     */
    get queueLength(): number;
    /**
     * Locks this mutex
     *
     * @throws {ConcurrencyInterruptedException} when the mutex is interrupted
     * @returns a promise when the lock has been set
     */
    lock(): Promise<void>;
    /**
     * Locks this mutex within a time limit
     *
     * Throws an error if the given time exceeds
     *
     * @param timeout maximum time (in ms) to lock
     * @throws {ConcurrencyExceedTimeoutException} when the time limit exceeds
     * @throws {ConcurrencyInterruptedException} when the mutex is interrupted
     * @returns a promise when the lock has been set
     */
    tryLock(timeout: number): Promise<void>;
    /**
     * Unlocks this mutex
     */
    unlock(): void;
    /**
     * Interrupts all awaiting "Threads" with an [exception]{@link ConcurrencyInterruptedException}.
     *
     * @param reason The reason why this mutex is being interrupted
     */
    interrupt(reason: unknown): void;
}
